<?php

namespace Drupal\index_now\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use GuzzleHttp\ClientInterface;

/**
 * Index now class.
 */
class IndexNow implements IndexNowInterface {

  use StringTranslationTrait;

  /**
   * The Index now default engine.
   *
   * @var string
   */
  protected $defaultEngine;

  /**
   * The HTTP client service.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The Index Now key manager service.
   *
   * @var \Drupal\index_now\Service\IndexNowKeyManagerInterface
   */
  protected $indexNowKeyManager;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * IndexNow constructor.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\index_now\Service\IndexNowKeyManagerInterface $index_now_key_manager
   *   The Index Now key manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_channel_factory
   *   The logger service.
   */
  public function __construct(
    ClientInterface $http_client,
    ConfigFactoryInterface $config_factory,
    IndexNowKeyManagerInterface $index_now_key_manager,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_channel_factory
  ) {
    $this->httpClient = $http_client;
    $this->indexNowKeyManager = $index_now_key_manager;
    $this->defaultEngine = $config_factory
      ->getEditable('index_now.settings')
      ->get('default_engine') ?? 'bing';
    $this->logger = $logger_channel_factory->get('index_now');
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public function sendRequest(string $page_url) {
    $api_key = $this->indexNowKeyManager->getKey();
    if (empty($api_key)) {
      return;
    }

    try {
      $url = $this->buildUrl($page_url, $api_key);
      $url = urldecode($url->toString());
      $request = $this->httpClient->request('GET', $url);
      if (!in_array($request->getStatusCode(), [200, 202])) {
        $warning = "Index Now returned an error: %error";
        $this->messenger->addWarning($this->t($warning, ['%error' => $request->getReasonPhrase()]));
        $this->logger->warning($warning, ['%error' => $request->getReasonPhrase()]);
        return;
      }
      $this->logger->info("%page_url has been indexed now.", ['%page_url' => $page_url]);
    }
    catch (\Exception $e) {
      $warning = "Index Now returned an error: %error";
      $this->messenger->addWarning($this->t($warning, ['%error' => $e->getMessage()]));
      $this->logger->warning($warning, ['%error' => $e->getMessage()]);
    }
  }

  /**
   * Build the search engine URL we want to hit to index now.
   *
   * @param string $page_url
   *   The page we want to index now on the search engines.
   * @param string $api_key
   *   The API key used to prove ownership of the host.
   *
   * @return \Drupal\Core\Url
   *   The search engine URL we want to hit to index now.
   */
  protected function buildUrl(string $page_url, string $api_key) {
    $engine_url = self::ENDPOINTS[$this->defaultEngine] ?? '';

    if (empty($engine_url)) {
      throw new \Exception('invalid Index Now engine URL.');
    }

    $endpoint = $engine_url . "/indexnow";
    $api_key_location = Url::fromUri('base:index_now_api_key_' . $api_key . '.txt');
    $api_key_location->setAbsolute();

    $url = Url::fromUri($endpoint);
    $url->setOption('query', [
      'url' => $page_url,
      'key' => $api_key,
      'keyLocation' => $api_key_location->toString(),
    ]);

    return $url;
  }

}
